module PlottingFunctions

export animate_ILC_multimode
export animate_ILC_multimode_talk

using ..Experiments
using ..ILCProblems

using NamedTrajectories

using CairoMakie

function animate_ILC_multimode(
    prob::ILCProblem,
    path::String;
    fps=5,
    plot_transformed_controls=false,
    markersize=10,
)
    fig = Figure(resolution=(1200, 1500))

    color = :glasbey_bw_minc_20_n256

    iter = Observable(0)

    Ygoal_ax = Axis(fig[1, :];
        title=@lift("Y goal: iter = $($iter)"),
        xlabel=L"t",
    )
    ylims!(Ygoal_ax, [-0.1, 1.1])

    Ȳ_ax = Axis(fig[2, :];
        title=@lift("Ȳ: iter = $($iter)"),
        xlabel=L"t",
    )
    ylims!(Ȳ_ax, [-0.1, 1.1])

    Ygoal = hcat(prob.Ygoal.ys...)
    τs = prob.Ygoal.τs

    series!(Ygoal_ax, τs, Ygoal;
        color=color,
        markersize=markersize
    )

    # axislegend(Ygoalax; position=:lb)

    Ȳ₁ = hcat(prob.Ȳs[1].ys...)

    Ȳ_sp = series!(Ȳ_ax, τs, Ȳ₁;
        color=color,
        markersize=markersize
    )

    # axislegend(Ȳax; position=:lb)

    ΔY_ax = Axis(fig[3, :];
        title=@lift("ΔY: iter = $($iter)"),
        xlabel=L"t"
    )

    ΔY₁ = Ȳ₁ - Ygoal
    autolimits!(ΔY_ax)

    ΔY_sp = series!(ΔY_ax, τs, ΔY₁;
        color=color,
        markersize=markersize
    )

    # axislegend(ΔYax; position=:lb)

    A_ax = Axis(fig[4, :]; title=@lift("a(t): iter = $($iter)"), xlabel=L"t")
    max_bound = maximum(prob.Ẑ.bounds.a)
    ylims!(A_ax, [-max_bound, max_bound])

    A₁ = prob.As[1]
    ts = times(prob.Ẑ)

    A_sp = series!(A_ax, ts, A₁;
        labels=["a_$j" for j = 1:prob.Ẑ.dims.a]
    )

    if plot_transformed_controls
        A_trans_ax = Axis(fig[5, :]; title=@lift("a(t) quantized: iter = $($iter)"), xlabel=L"t")

        A_trans_1 = prob.experiment.controls_transform(A₁)

        A_trans_sp = series!(A_trans_ax, ts, A_trans_1;
            labels=["a_$j" for j = 1:prob.Ẑ.dims.a]
        )
    end

    record(fig, path, 1:length(prob.Ȳs); framerate=fps) do i
        iter[] = i

        Ȳ = hcat(prob.Ȳs[i].ys...)
        ΔY = Ȳ - Ygoal
        A = prob.As[i]

        Ȳ_sp[2] = Ȳ
        ΔY_sp[2] = ΔY
        A_sp[2] = A
        autolimits!(ΔY_ax)
        autolimits!(A_ax)
        if plot_transformed_controls
            A_trans_sp[2] = prob.experiment.controls_transform(A)
        end
    end
end

function animate_ILC_multimode_talk(
    prob::ILCProblem,
    path::String;
    fps=5,
    plot_transformed_controls=false,
    markersize=10,
)
    fig = Figure(resolution=(1000, 800))

    color = :glasbey_bw_minc_20_n256

    iter = Observable(0)

    ax_count = 0

    loss_ax = Axis(fig[1, :];
        title="Loss",
        xlabel="iter",
    )
    lines!(loss_ax, prob.loss; color=:blue)
    ax_count += 1

    # Ygoal_ax = Axis(fig[1, :];
    #     title=@lift("Y goal: iter = $($iter)"),
    #     xlabel=L"t",
    # )
    # ylims!(Ygoal_ax, [-0.1, 1.1])

    # Ȳ_ax = Axis(fig[2, :];
    #     title=@lift("Ȳ: iter = $($iter)"),
    #     xlabel=L"t",
    # )
    # ylims!(Ȳ_ax, [-0.1, 1.1])

    Ygoal = hcat(prob.Ygoal.ys...)
    τs = prob.Ygoal.τs

    # series!(Ygoal_ax, τs, Ygoal;
    #     color=color,
    #     markersize=markersize
    # )

    # axislegend(Ygoalax; position=:lb)

    Ȳ₁ = hcat(prob.Ȳs[1].ys...)

    # Ȳ_sp = series!(Ȳ_ax, τs, Ȳ₁;
    #     color=color,
    #     markersize=markersize
    # )

    # axislegend(Ȳax; position=:lb)

    ΔY_ax = Axis(fig[ax_count + 1, :];
        title=@lift("ΔY: iter = $($iter)"),
        xlabel=L"t"
    )
    ax_count += 1

    ΔY₁ = Ȳ₁ - Ygoal
    autolimits!(ΔY_ax)

    ΔY_sp = series!(ΔY_ax, τs, ΔY₁;
        color=color,
        markersize=markersize
    )

    # axislegend(ΔYax; position=:lb)

    # A_ax = Axis(fig[4, :]; title=@lift("a(t): iter = $($iter)"), xlabel=L"t")
    # max_bound = maximum(prob.Ẑ.bounds.a)
    # ylims!(A_ax, [-max_bound, max_bound])

    A₁ = prob.As[1]
    ts = times(prob.Ẑ)

    # A_sp = series!(A_ax, ts, A₁;
    #     labels=["a_$j" for j = 1:prob.Ẑ.dims.a]
    # )

    if plot_transformed_controls
        A_trans_ax = Axis(fig[ax_count + 1, :]; title=@lift("a(t) quantized: iter = $($iter)"), xlabel=L"t")
        ax_count += 1

        A_trans_1 = prob.experiment.controls_transform(A₁)

        A_trans_sp = series!(A_trans_ax, ts, A_trans_1;
            labels=["a_$j" for j = 1:prob.Ẑ.dims.a]
        )
    end

    record(fig, path, 1:length(prob.Ȳs); framerate=fps) do i
        iter[] = i

        Ȳ = hcat(prob.Ȳs[i].ys...)
        ΔY = Ȳ - Ygoal
        A = prob.As[i]

        # Ȳ_sp[2] = Ȳ
        ΔY_sp[2] = ΔY
        # A_sp[2] = A
        autolimits!(loss_ax)
        # autolimits!(A_ax)
        if plot_transformed_controls
            A_trans_sp[2] = prob.experiment.controls_transform(A)
        end
    end
end



end
